unit mainunit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Buttons, DirectShow, ActiveX, ComCtrls;

Const
  dsEventMessage                    = WM_APP+12345;  // Unique event identifier
  dsEventInstance  : Integer        = 54321;         // Unique event identifier
  dsEventsOn       : Integer        = 0;
  dsEventsOff      : Integer        = 1;
  VolumeTable      : Array[0..7]  of Integer = (-10000,-4800,-2400,-1200,-600,-300,-150,0);

type
  TMainForm = class(TForm)
    ControlPanel: TPanel;
    VideoPanel: TPanel;
    PlayButton: TSpeedButton;
    StopButton: TSpeedButton;
    QuitButton: TSpeedButton;
    OpenButton: TSpeedButton;
    OpenDialog: TOpenDialog;
    SeekTimer: TTimer;
    TimeLinePanel: TPanel;
    TimeLineImage: TImage;
    VolumeBar: TTrackBar;
    procedure QuitButtonClick(Sender: TObject);
    procedure OpenButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure PlayButtonClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormCanResize(Sender: TObject; var NewWidth,
      NewHeight: Integer; var Resize: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure SeekTimerTimer(Sender: TObject);
    procedure TimeLineImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure TimeLineImageMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure VolumeBarChange(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    procedure dsEventPop(var M : TMessage); message dsEventMessage;
  public
    { Public declarations }
    function  CreateMovieInterface : Boolean;
    procedure DestroyMovieInterface;
    procedure SetWindowPos(pLeft,pTop,pWidth,pHeight : Integer);
    procedure ResetWindowPos;
    procedure SetCurrentPosition(mPos : Int64);
    function  GetCurrentPosition : Int64;
    procedure UpdateTimeLine;
    procedure SetVolume(vLevel : Integer);
  end;

Const
  stClosed   = 0;
  stPlaying  = 1;
  stStopped  = 2;
  stPaused   = 3;
  stOpen     = 4;

var
  MainForm         : TMainForm;

  // DirectShow Interfaces
  dsGraphBuilder   : IGraphBuilder    = NIL; // Used to open the file and render the filters
  dsMediaControl   : IMediaControl    = NIL; // Play, Stop, Pause.
  dsMediaSeeking   : IMediaSeeking    = NIL; // Set position.
  dsBasicAudio     : IBasicAudio      = NIL; // Volume/Balance control.
  dsBasicVideo     : IBasicVideo2     = NIL; // Video source size
  dsVideoWindow    : IVideoWindow     = NIL; // Window positioning
  dsMediaEventEx   : IMediaEventEx    = NIL; // Event handling

  // Program vars
  plState          : Integer = stClosed;     // Play State
  MediaLength      : Int64   = -1;           // Length of open media
  MediaPosition    : Int64   =  0;           // Used to draw the time bar

implementation

{$R *.dfm}

function TMainForm.CreateMovieInterface : Boolean;
begin
  Result := False;

  // Init COM Interface
  If failed(CoInitialize(NIL)) then Exit;

  // Create DirectShow Graph
  if failed(CoCreateInstance(CLSID_FilterGraph,NIL, CLSCTX_INPROC,IID_IGraphBuilder,dsGraphBuilder)) then exit;

  // Get the IMediaControl Interface
  if failed(dsGraphBuilder.QueryInterface(IID_IMediaControl,dsMediaControl)) then exit;

  // Get the IMediaSeeking Interface
  if failed(dsGraphBuilder.QueryInterface(IID_IMediaSeeking,dsMediaSeeking)) then exit;

  // Get the IBasicAudio Interface
  if failed(dsGraphBuilder.QueryInterface(IID_IBasicAudio,dsBasicAudio)) then exit;

  // Get the IBasicWindow Interface
  if failed(dsGraphBuilder.QueryInterface(IID_IBasicVideo,dsBasicVideo)) then exit;

  // Get the IVideoWindow Interface
  if failed(dsGraphBuilder.QueryInterface(IID_IVideoWindow,dsVideoWindow)) then exit;

  // Get the IMediaEventEx Interface
  if failed(dsGraphBuilder.QueryInterface(IID_IMediaEventEx,dsMediaEventEx)) then exit;

  // Attach the IMediaEventEx interface to the main window
  dsMediaEventEx.SetNotifyWindow(MainForm.Handle,dsEventMessage,dsEventInstance);

  // Enable IMediaEventEx notifications
  dsMediaEventEx.SetNotifyFlags(dsEventsOn);

  Result := True;
end;

procedure TMainForm.DestroyMovieInterface;
var
  I               : Integer;
  dsFilterEnum    : IEnumFilters;
  dsFilterCount   : Integer;
  dsFetched       : uLong;
  dsFilters       : Array[0..49] of IBaseFilter;
begin
  // Stop playback if we're trying to open a movie if one is already loaded.
  If Assigned(dsMediaControl) then dsMediaControl.Stop;

  // Remove all filters
  If Assigned(dsGraphBuilder) then
  Begin
    dsFilterEnum    := nil;
    dsFilterCount   := 0;
    I               := 0;
    If dsGraphBuilder.EnumFilters(dsFilterEnum) = S_OK then
    Begin
      // Count the currently active filters
      While (dsFilterEnum.Skip(1) = S_OK) do inc(dsFilterCount);
      dsFilterEnum.Reset;
      dsFetched := 0;

      // Load the filters into an array
      While (dsFilterEnum.Next(1, dsFilters[I],Addr(dsFetched)) = S_OK) do Inc(I);

      If Assigned(dsFilterEnum) then dsFilterEnum := NIL;

      // Remove the filters from the graph
      If dsFilterCount > 0 then For I := 0 to dsFilterCount-1 do
      begin
        dsGraphBuilder.RemoveFilter(dsFilters[I]);
        dsFilters[I] := nil;
      end;
    End;
  End;

  // Detach ownership to stop events
  If Assigned(dsVideoWindow) then
  Begin
    dsVideoWindow.put_Visible(false);
    dsVideoWindow.put_Owner(0);
  End;

  // Release all remaining pointers
  if Assigned(dsMediaControl)   then dsMediaControl  := NIL;
  if Assigned(dsMediaSeeking)   then dsMediaSeeking  := NIL;
  if Assigned(dsBasicAudio)     then dsBasicAudio    := NIL;
  if Assigned(dsBasicVideo)     then dsBasicVideo    := NIL;
  if Assigned(dsVideoWindow)    then dsVideoWindow   := NIL;
  if Assigned(dsMediaEventEx)   then dsMediaEventEx  := NIL;
  if Assigned(dsGraphBuilder)   then dsGraphBuilder  := NIL;

  // Clean up the COM interface
  CoUninitialize;

  plState := stClosed;
end;

procedure TMainForm.dsEventPop(var M : TMessage);
var
  dsEventCode   : Integer;
  dsEventParam1 : Integer;
  dsEventParam2 : Integer;
begin
  inherited;
  If M.lParam = dsEventInstance then
  Begin
    If (Application.Terminated = False) and (Assigned(dsMediaEventEx) = True) then
      If dsMediaEventEx.GetEvent(dsEventCode,dsEventParam1,dsEventParam2,10) <> E_Abort then
    Begin
      Case dsEventCode of
        EC_COMPLETE     : Begin
                            StopButton.Click;
                            plState := stStopped;
                          End;
      End;

      // Free the event from memory
      dsMediaEventEx.FreeEventParams(dsEventCode,dsEventParam1,dsEventParam2);
    End;
  End;
end;

procedure TMainForm.SetWindowPos(pLeft,pTop,pWidth,pHeight : Integer);
begin
  If (Assigned(dsVideoWindow)) and (plState <> stClosed) then
  Begin
    dsVideoWindow.SetWindowPosition(pLeft,pTop,pWidth,pHeight);
  End;
end;

procedure TMainForm.ResetWindowPos;
begin
  SetWindowPos(0,0,VideoPanel.Width,VideoPanel.Height);
end;

procedure TMainForm.SetCurrentPosition(mPos : Int64);
begin
  If Assigned(dsMediaSeeking) then
  Begin
    dsMediaSeeking.SetPositions(mPos,AM_SEEKING_AbsolutePositioning,mPos,AM_SEEKING_NoPositioning);
    MediaPosition := mPos;
  End;
end;

function TMainForm.GetCurrentPosition : Int64;
var
  CurPos  : Int64;
  StopPos : Int64; // Not really used by this program
begin
  // Default result in case of failure
  Result := -1;
  If Assigned(dsMediaSeeking) then
  Begin
    If dsMediaSeeking.GetPositions(CurPos,StopPos) = S_OK then
    Begin
      Result := CurPos;
    End;
  End;
end;

procedure TMainForm.QuitButtonClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.OpenButtonClick(Sender: TObject);
var
  FileName        : String;
  UnicodeFileName : Array[0..(MAX_PATH*2)-1] of Char;
begin
  If OpenDialog.Execute = True then
  Begin
    Filename := OpenDialog.FileName;
    If FileExists(FileName) = True then
    Begin
      // First clean the old movie interface
      DestroyMovieInterface;

      // In case we can't seek this file
      MediaPosition := 0;

      // Now create a new movie interface
      If CreateMovieInterface = True then
      Begin
        // Get filename in UNICODE
        MultiByteToWideChar(CP_ACP,0,PChar(FileName),-1,@UnicodeFileName,MAX_PATH);

        // Render the file filters
        If dsGraphBuilder.RenderFile(@UnicodeFileName,nil) = S_OK then
        Begin
          SetVolume(VolumeBar.Position);

          plState := stOpen;

          If Assigned(dsMediaSeeking) then
          Begin
            dsMediaSeeking.GetDuration(MediaLength);
          End;

          // Set our form as the owner
          dsVideoWindow.put_Owner(MainForm.Handle);
          // Set the video window messages (mouse/keyboard) to be routed to our form
          dsVideoWindow.put_MessageDrain(MainForm.Handle);
          // Set the video window to be a child window of our form.
          dsVideoWindow.put_WindowStyle(WS_CHILD or WS_CLIPSIBLINGS or WS_CLIPCHILDREN);

          // Set the inital window position
          ResetWindowPos;

          // Seek the video to the start and pause.
          StopButton.Click;
        End
          else
        Begin
          ShowMessage('Unable to render the file');
          DestroyMovieInterface;
        End;
      End
      Else ShowMessage('Unable to create DirectShow Interface');
    End;
  End;
end;

procedure TMainForm.StopButtonClick(Sender: TObject);
begin
  If (plState <> stClosed) and (plState <> stStopped) then
  Begin
    If Assigned(dsMediaControl) then
    Begin
      // Stop Playback
      dsMediaControl.Stop;
      // Seek to First Frame
      SetCurrentPosition(0);
      // Pause Playback (brings up first image)
      dsMediaControl.Pause;
      plState := stStopped;
      PlayButton.Caption    := 'Play';
      UpdateTimeLine;
    End;
  End;
end;

procedure TMainForm.PlayButtonClick(Sender: TObject);
begin
  If Assigned(dsMediaControl) and (plState <> stClosed) then
  Begin
    // Pause if playing, Play if Stopped or Paused.
    If (plState = stPaused) or (plState = stStopped) then
    Begin
      dsMediaControl.Run;
      plState := stPlaying;
      PlayButton.Caption := 'Pause';
    End
      else
    Begin
      dsMediaControl.Pause;
      plState := stPaused;
      PlayButton.Caption := 'Play';
    End;
  End;
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  ResetWindowPos;
end;

procedure TMainForm.FormCanResize(Sender: TObject; var NewWidth,
  NewHeight: Integer; var Resize: Boolean);
begin
  If (NewHeight < 359) then NewHeight := 359;
  If (NewWidth  < 408) then NewWidth  := 408;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  DestroyMovieInterface;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  // Smoother form draws
  MainForm.DoubleBuffered := True;

  // Create the bitmap used for the timeline
  TimeLineImage.Picture.Bitmap.Width       := TimeLineImage.Width;
  TimeLineImage.Picture.Bitmap.Height      := TimeLineImage.Height;
  TimeLineImage.Picture.Bitmap.PixelFormat := pf24bit;
  UpdateTimeLine;
end;

procedure TMainForm.UpdateTimeLine;
begin
  // Clear the TimeLine;
  TimeLineImage.Picture.Bitmap.Canvas.Brush.Color := clWhite;
  TimeLineImage.Picture.Bitmap.Canvas.FillRect(Rect(0,0,TimeLineImage.Width,TimeLineImage.Height));

  // Fill in the current position
  TimeLineImage.Picture.Bitmap.Canvas.Brush.Color := clBlue;
  TimeLineImage.Picture.Bitmap.Canvas.FillRect(Rect(0,0,Trunc(MediaPosition/(MediaLength / 100)),TimeLineImage.Height));
end;

procedure TMainForm.SetVolume(vLevel : Integer);
begin
  If Assigned(dsBasicAudio) then dsBasicAudio.Put_Volume(VolumeTable[vLevel]);
end;

procedure TMainForm.SeekTimerTimer(Sender: TObject);
begin
  If plState <> stClosed then
  Begin
    MediaPosition := GetCurrentPosition;
    UpdateTimeLine;
  End;
end;

procedure TMainForm.TimeLineImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  // Relay the even to the mouse move since they do the same thing
  TimeLineImageMouseMove(Sender,Shift,X,Y);
end;

procedure TMainForm.TimeLineImageMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  // Set to the clicked position, only if left mouse button is clicked
  If Shift = [ssLeft] = True then
  Begin
    SetCurrentPosition(Trunc((MediaLength/100)*X));
    UpdateTimeLine;
  End;  
end;

procedure TMainForm.VolumeBarChange(Sender: TObject);
begin
  If plState <> stClosed then SetVolume(VolumeBar.Position);
end;

procedure TMainForm.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  PlayButton.Click;
end;

end.
